﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: HPS")

L:RegisterTranslations("enUS", function() return {
	["Healing per Second"] = true,
	["HPS"] = true,
} end)

--Chinese Translate by Ananhaid(NovaLOG)@CWDG
--CWDG site: http://Cwowaddon.com
L:RegisterTranslations("zhCN", function() return {
	["Healing per Second"] = "每秒治疗",
	["HPS"] = "每秒治疗(HPS)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Healing per Second"] = "每秒治療",
	["HPS"] = "每秒治療(HPS)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing per Second"] = "초당 치유량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Healing per Second"] = "Soins par seconde",
	["HPS"] = "SPS",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local seconds = nil
local healingDone = nil
local moduleName = "Healing per Second"
local isInCombat = {}
local combatStartTime = {}
local lastHealTime = {}

local COMBAT_TIMEOUT = 4

local roster = AceLibrary("Roster-2.1")

local parserEvent = {
	eventType = "Heal",
	sourceID_not = false,
}

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 56720 $"):sub(12, -3)) or 1
mod.version = 5
mod.hidePercentage = true
mod.violationCategory = "healing"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterParserEvent(parserEvent, "OnParserEvent")
	mod:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	mod:ScheduleRepeatingEvent( self.CheckPartyCombat, 1, self )
end

function mod:OnReset()
	if type(seconds) == "table" then
		seconds = mod:del(seconds)
	end
	seconds = mod:new()
	
	if type(healingDone) == "table" then
		healingDone = mod:del(healingDone)
	end
	healingDone = mod:new()

	for k in pairs(combatStartTime) do
		combatStartTime[k] = nil
	end
	for k in pairs(lastHealTime) do
		lastHealTime[k] = nil
	end
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Healing per Second"] end
function mod:GetShortDisplayName() return L["HPS"] end

function mod:OnRemoteData(name, input)
	if name and input then
		self.dirty = (input ~= (self.data[name] or 0))
		self.data[name] = input
	end
end

function mod:OnData(name, input)
	if not self:IsLocal(name) and self.data[name] then
		return
	end	
	self.data[name] = nil
	self.dirty = true
end

function mod:IsDirty()
	return self.dirty or next(isInCombat)
end

function mod:IsBroadcastDirty( charName )
	if not charName then
		charName = self.playerName
	end
	return ( self.broadcast[charName] ~= self:GetData(charName) ) or isInCombat[charName]
end

local function getDuration(name)
	local duration = seconds[name] or 0
	if combatStartTime[name] then
		duration = duration + math.max((lastHealTime[name] or GetTime()) - combatStartTime[name], 1.5)
	end
	return math.max(duration, 1.5)
end

local function calculateData(name)
	return math.floor((healingDone[name] or 0) / getDuration(name))
end

local moduleData = {}
function mod:GetData(name)
	if name then
		return self.data[name] or calculateData(name)
	else
		for k, v in pairs( moduleData ) do
			moduleData[k] = nil
		end
		for name in pairs(healingDone) do
			moduleData[name] = calculateData(name)
		end
		for name, value in pairs(self.data) do
			moduleData[name] = value 
		end
		return moduleData
	end
end

function mod:GetBroadcastData( charName )
	if not charName then
		charName = self.playerName
	end
	self.broadcast[charName] = self:GetData(charName)
	return self.broadcast[charName]
end

function mod:UpdateHPS(name)
	self:OnData(name, true)
end

function mod:ResetHPSTimer(name)
	if combatStartTime[name] then
		seconds[name] = getDuration(name)
		
		combatStartTime[name] = nil
		lastHealTime[name] = nil
		
		self:UpdateHPS(name)
	end
end

function mod:CheckHPSTimer(name)
	if lastHealTime[name] and lastHealTime[name] + COMBAT_TIMEOUT < GetTime() then
		self:ResetHPSTimer(name)
	end
end

local function getOverheal( unit, amount )
	local overheal = 0
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			overheal = amount - d
		end
	end
	return overheal
end

function mod:OnParserEvent( info )
	local name, target, amount = self:GetSourceVictimAmount(info)

	if not name or not target or amount == 0 or not self:AcceptDataFor( name ) then
		return
	end

	local targetid = roster:GetUnitIDFromName( target )
	amount = amount - getOverheal( targetid, amount )
	
	self:CheckHPSTimer()
	
 	local newTime = GetTime()
	combatStartTime[name] = combatStartTime[name] or newTime
	lastHealTime[name] = newTime
	healingDone[name] = ( healingDone[name] or 0 ) + amount	
	
	mod:UpdateHPS(name)
end

-----------------------------------------------------------------------
-- Combat status watching
-----------------------------------------------------------------------

function mod:PLAYER_REGEN_ENABLED()
	mod:ResetHPSTimer( playerName )
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] then
			if not UnitAffectingCombat( unit.unitid ) then
				isInCombat[name] = nil
				mod:ResetHPSTimer( name )
			else
				mod:CheckHPSTimer(name)			
			end
		elseif UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
